from common import download_file_local, delete_local_file
from dotenv import load_dotenv
import requests
import os
import uuid 
import boto3
from botocore.exceptions import NoCredentialsError, ClientError
from urllib.parse import urlparse
from datetime import datetime
import logging

# Configure the logger
logging.basicConfig(level=logging.INFO)  # Set the logging level to INFO or your preferred level
logger = logging.getLogger(__name__)  # Create a logger object for your module (__name__ will use the current module's name)

def s3_environment():

    load_dotenv()  # This loads the environment variables from .env
    S3_ACCESS_KEY_ID = os.getenv("S3_ACCESS_KEY_ID")
    S3_SECRET_ACCESS_KEY = os.getenv("S3_SECRET_ACCESS_KEY")
    S3_URL = os.getenv("S3_URL", 'https://eu-central-1.s3.amazonaws.com/getaiir')
    S3_URL = os.getenv("S3_URL", 'https://getaiir.s3.eu-central-1.amazonaws.com')
    S3_CUBE_PUBLIC = os.getenv("S3_CUBE_PUBLIC", 'getaiir/public/')
    S3_BUCKET_NAME = os.getenv("S3_BUCKET_NAME", 'getaiir')
    S3_CUBE_NAME = os.getenv("S3_CUBE_NAME", 'eu-central-1')

    # Create a dictionary to store the values
    s3_config_dict = {
        "S3_ACCESS_KEY_ID": S3_ACCESS_KEY_ID,
        "S3_SECRET_ACCESS_KEY": S3_SECRET_ACCESS_KEY,
        "S3_URL": S3_URL,
        "S3_CUBE_NAME": S3_CUBE_NAME,
        "S3_CUBE_PUBLIC": S3_CUBE_PUBLIC,
        "S3_BUCKET_NAME": S3_BUCKET_NAME
    }

    return s3_config_dict

def s3_list_files_in_folder(folder_name="", s3_config_dict=None):

    '''
    Lists all object in a folder_name from the bucket_name
    '''
    access_key=s3_config_dict['S3_ACCESS_KEY_ID']
    secret_key=s3_config_dict['S3_SECRET_ACCESS_KEY']
    bucket_name=s3_config_dict['S3_BUCKET_NAME']

    s3 = boto3.client('s3', aws_access_key_id=access_key, aws_secret_access_key=secret_key)
    
    prefix = folder_name + '/' if folder_name else None
    response = s3.list_objects_v2(Bucket=bucket_name, Prefix=prefix)
  
    # Check if objects were listed successfully
    if 'Contents' in response:
        objects = response['Contents']
        for obj in objects:
            print("Object Key:", obj['Key'])
    else:
        print("No objects found in the bucket.")

    return objects

def s3_upload_local_file(file_name="", folder_name=None, s3_config_dict=None, make_public=False):
    '''
    Uploads from file name to the folder_name
    file_name is local, must not be a file_path
    
    '''

    if s3_config_dict is None:
        raise ValueError("s3_config_dict is required for this function.")
    
    # Extract relevant values from heroku_config_dict
    access_key=s3_config_dict['S3_ACCESS_KEY_ID']
    secret_key=s3_config_dict['S3_SECRET_ACCESS_KEY']
    bucket_name=s3_config_dict['S3_BUCKET_NAME']
    s3_url=s3_config_dict['S3_URL']
    cube_name=s3_config_dict['S3_CUBE_NAME']
    cube_public=s3_config_dict['S3_CUBE_PUBLIC']

    s3 = boto3.client('s3', aws_access_key_id=access_key, aws_secret_access_key=secret_key)

    # Ensure folder ends with a '/' if it's not empty
    if folder_name is None:
        folder_name = ''
    elif not folder_name.endswith('/'):
        folder_name += '/'

    object_key = f"{folder_name}{file_name}"

    # print(f"Object key: {object_key}")

    try:
        response = s3.upload_file(file_name, bucket_name, object_key)

        # Set the ACL to public-read if specified
        if make_public:
            s3.put_object_acl(Bucket=bucket_name, Key=object_key, ACL='public-read')

        print(f"File '{object_key}' uploaded successfully to bucket '{bucket_name}'.")
        return object_key
    except Exception as e:
        print(f"An error occurred while uploading the file: {str(e)}")
        return None
    
def s3_delete_file(object_key, s3_config_dict):
    access_key = s3_config_dict['S3_ACCESS_KEY_ID']
    secret_key = s3_config_dict['S3_SECRET_ACCESS_KEY']
    bucket_name = s3_config_dict['S3_BUCKET_NAME']

    s3 = boto3.client('s3', aws_access_key_id=access_key, aws_secret_access_key=secret_key)

    try:
        response = s3.delete_object(Bucket=bucket_name, Key=object_key)
        print(f"File '{object_key}' deleted successfully from bucket '{bucket_name}'.")
        return response
    except Exception as e:
        print(f"An error occurred while deleting the file: {str(e)}")
        return None
   
def s3_upload_file_from_url(file_url="", folder_name="", s3_config_dict=None, bnewname=False, make_public=False):
    
    # Download the file to a local path
    file_path = download_file_local(file_url)

    # print(file_path)

    if bnewname:
        # Extract the file extension from the URL
        parsed_url = urlparse(file_url)
        file_extension = os.path.splitext(os.path.basename(parsed_url.path))[1]
        # print(file_extension)

        # Generate a timestamp in the format YYYYMMDDHHMMSS
        timestamp = datetime.now().strftime("%Y%m%d%H%M%S")

        # Generate a UUID
        generated_uuid = uuid.uuid4()

        # Convert the UUID to a hexadecimal string and truncate it to the desired length
        shortened_uuid = str(generated_uuid.hex)[:8]  # Example: Truncate to 8 characters

        # Create a new filename using the shortened UUID
        new_filename = f"{timestamp}_{shortened_uuid}{file_extension}"

        # Rename the local file with the new filename
        new_file_path = os.path.join(os.path.dirname(file_path), new_filename)
        os.rename(file_path, new_file_path)

        # Upload the file with the new filename
        object_key = s3_upload_local_file(file_name=new_file_path, folder_name=folder_name, s3_config_dict=s3_config_dict, make_public=make_public)

        # Delete the local file (both the original and renamed versions)
        delete_local_file(new_file_path)
    else:
        # Upload the file with its original name
        object_key = s3_upload_local_file(file_name=file_path, folder_name=folder_name, s3_config_dict=s3_config_dict)

        # Delete the local file
        delete_local_file(file_path)

    s3_url =s3_config_dict['S3_URL']

    file_url = s3_url + '/' + object_key

    return file_url